@@ -2,6 +2,8 @@ import itertools
 import unittest
 
 from testify import assert_equal
+from testify import assert_not_equal
+from testify import assert_in
 from testify import class_setup
 from testify import class_setup_teardown
 from testify import class_teardown
@@ -12,6 +14,7 @@ from testify import setup_teardown
 from testify import teardown
 from testify import TestCase
 from testify import test_runner
+from testify import suite
 
 
 class TestMethodsGetRun(TestCase):
@@ -290,6 +293,15 @@ class TestFixtureMixinsGetRun(TestCase, FixtureMixin):
         assert self.foo_ran
 
 
+class RedefinedFixtureWithNoDecoratorTest(TestCase, FixtureMixin):
+    def set_attr(self):
+        pass
+
+    def test_foo(self):
+        # set_attr shouldn't have run because it's no longer decorated
+        assert not hasattr(self, 'foo')
+
+
 class TestSubclassedCasesWithFeatureMixinsGetRun(TestFixtureMixinsGetRun):
     pass
 
@@ -497,10 +509,139 @@ class LetWithLambdaTest(TestCase):
         assert_equal(self.counter.next(), 0)
         assert_equal(self.counter.next(), 1)
 
-class LetWithSubclassTest(TestCase):
+class LetWithSubclassTest(LetWithLambdaTest):
     """Test that @let is inherited correctly."""
     pass
 
 
+class CallbacksGetCalledTest(TestCase):
+    def test_class_fixtures_get_reported(self):
+        """Make a test case, register a bunch of callbacks for class fixtures on it, and make sure the callbacks are all run in the right order."""
+        class InnerTestCase(TestCase):
+            def classSetUp(self):
+                pass
+
+            def classTearDown(self):
+                pass
+
+            @class_setup_teardown
+            def __class_setup_teardown(self):
+                yield
+
+            def test_things(self):
+                pass
+
+        inner_test_case = InnerTestCase()
+        events = (
+            TestCase.EVENT_ON_RUN_TEST_METHOD,
+            TestCase.EVENT_ON_COMPLETE_TEST_METHOD,
+            TestCase.EVENT_ON_RUN_CLASS_SETUP_METHOD,
+            TestCase.EVENT_ON_COMPLETE_CLASS_SETUP_METHOD,
+            TestCase.EVENT_ON_RUN_CLASS_TEARDOWN_METHOD,
+            TestCase.EVENT_ON_COMPLETE_CLASS_TEARDOWN_METHOD,
+            TestCase.EVENT_ON_RUN_TEST_CASE,
+            TestCase.EVENT_ON_COMPLETE_TEST_CASE,
+        )
+
+        calls_to_callback = []
+        def make_callback(event):
+            def callback(result):
+                calls_to_callback.append((event, result['method']['name'] if result else None))
+            return callback
+
+        for event in events:
+            inner_test_case.register_callback(event, make_callback(event))
+
+        inner_test_case.run()
+
+        assert_equal(calls_to_callback, [
+            (TestCase.EVENT_ON_RUN_TEST_CASE, 'run'),
+
+            (TestCase.EVENT_ON_RUN_CLASS_SETUP_METHOD, 'classSetUp'),
+            (TestCase.EVENT_ON_COMPLETE_CLASS_SETUP_METHOD, 'classSetUp'),
+
+            (TestCase.EVENT_ON_RUN_CLASS_SETUP_METHOD, '__class_setup_teardown'),
+            (TestCase.EVENT_ON_COMPLETE_CLASS_SETUP_METHOD, '__class_setup_teardown'),
+
+            (TestCase.EVENT_ON_RUN_TEST_METHOD, 'test_things'),
+            (TestCase.EVENT_ON_COMPLETE_TEST_METHOD, 'test_things'),
+
+            (TestCase.EVENT_ON_RUN_CLASS_TEARDOWN_METHOD, '__class_setup_teardown'),
+            (TestCase.EVENT_ON_COMPLETE_CLASS_TEARDOWN_METHOD, '__class_setup_teardown'),
+
+            (TestCase.EVENT_ON_RUN_CLASS_TEARDOWN_METHOD, 'classTearDown'),
+            (TestCase.EVENT_ON_COMPLETE_CLASS_TEARDOWN_METHOD, 'classTearDown'),
+
+            (TestCase.EVENT_ON_COMPLETE_TEST_CASE, 'run'),
+        ])
+
+
+class TestCaseKeepsReferenceToResultsForTestMethod(TestCase):
+    def test_reference_to_results(self):
+        assert self.test_result
+
+
+class SuiteDecoratorTest(TestCase):
+
+    def test_suite_pollution_with_suites_attribute(self):
+        """Test if suite decorator modifies the object's attribute
+        objects instead of assigning a new object. Modifying _suite
+        attribute objects causes suite pollution in TestCases.
+
+        Here we test if the _suites attribute's id() remains the same
+        to verify suite decorator does not modify the object's
+        attribute object.
+        """
+
+        def function_to_decorate():
+            pass
+
+        function_to_decorate._suites = set(['fake_suite_1'])
+
+        suites_before_decoration = function_to_decorate._suites
+
+        function_to_decorate = suite('fake_suite_2')(function_to_decorate)
+
+        suites_after_decoration =  function_to_decorate._suites
+
+        assert_not_equal(
+            id(suites_before_decoration),
+            id(suites_after_decoration),
+            "suites decorator modifies the object's _suite attribute"
+        )
+
+
+class FailingTeardownMethodsTest(TestCase):
+
+    def test_class_with_two_failing_teardown_methods(self):
+
+        class ClassWithTwoFailingTeardownMethods(TestCase):
+
+            methods_ran = []
+
+            def test_method(self):
+                self.methods_ran.append("test_method")
+                assert False
+
+            @teardown
+            def first_teardown(self):
+                self.methods_ran.append("first_teardown")
+                assert False
+
+            @teardown
+            def second_teardown(self):
+                self.methods_ran.append("second_teardown")
+                assert False
+
+        inner_test_case = ClassWithTwoFailingTeardownMethods()
+        inner_test_case.run()
+        
+        assert_in("test_method", inner_test_case.methods_ran)
+        assert_in("first_teardown", inner_test_case.methods_ran)
+        assert_in("second_teardown", inner_test_case.methods_ran)
+
+
 if __name__ == '__main__':
     run()
+
+# vim: set ts=4 sts=4 sw=4 et:
